from cmtt.preprocessing.tokenizer import WordTokenizer

def search_word(corpus, word, tokenize = False, width = 10):
  """
    Returns number of instances and list of all instances of the target word in corpus,
    :param corpus: the corpus in which the target words is to be searched
    :type corpus: list or str
    :param word: the target word
    :type text: str
    :param tokenize: option to tokenize the corpus or not
    :type tokenize: bool
    :param width: number of words/characters in the line
    :type width: int
    :return: number of instances and list of all instances of the target word in corpus
    :rtype: int, list
  """

  if(tokenize):
    WordT = WordTokenizer()
    if(type(corpus) == list):
      corpus = ' '.join(corpus)
      corpus = WordT.tokenize(corpus)
    else:
      corpus = WordT.tokenize(corpus)

  instances = 0
  list_instances = []
  lwidth = width//2
  rwidth = width - width//2
  for i in range(0, len(corpus)):
    if(word.lower() == corpus[i].lower()):
      instances += 1
      list_instances.append(corpus[(i - lwidth) if (i - lwidth) >= 0 else 0 : (i + rwidth) if (i + rwidth) <= len(corpus) else len(corpus)])

  print("Found the following " + str(instances) + " instances of " + word + " in corpus:")
  for i in list_instances:
    print(' '.join(i))

  return instances, list_instances


import unittest
from cmtt.data import *
from cmtt.preprocessing import *

class CMTTTestCase(unittest.TestCase):
  def setUp(self):
    self.load_url = load_url
    self.list_dataset_keys = list_dataset_keys
    self.WordT = WordTokenizer()
    self.WordpieceT = Wordpiece_tokenizer()
    self.WhitespaceT = WhitespaceTokenizer()

  def test_data_load_url_1(self):
    """Test data load_url function 1"""
    result = self.load_url('https://world.openfoodfacts.org/api/v0/product/5060292302201.json')
    self.assertEqual(result['code'], "5060292302201")

  def test_data_load_url_2(self):
    """Test data load_url function 2"""
    result = self.load_url('https://gist.githubusercontent.com/rnirmal/e01acfdaf54a6f9b24e91ba4cae63518/raw/6b589a5c5a851711e20c5eb28f9d54742d1fe2dc/datasets.csv')
    self.assertEqual(result['about'][20], "Tate Collection metadata")
    self.assertEqual(len(result['about']), 61)

  def test_list_dataset_keys(self):
    """Test list_dataset_keys function"""
    keys = self.list_dataset_keys()
    self.assertEqual(keys, ['url', 'id', 'name', 'format', 'language', 'task', 'credits', 'source', 'last_modified'])

  def test_whitespace_tokenize(self):
    """Test whitespace tokenize function"""
    text = "Hello world! This a python code. Let's activate"
    result = self.WhitespaceT.tokenize(text)
    self.assertEqual(len(result), 8)
  
  def test_word_tokenize(self):
    """Test word tokenize function"""
    text = "This Python interpreter is in a conda environment, but the environment has not been activated.  Libraries may fail to load.  To activate this environment"
    result = self.WordT.tokenize(text)
    self.assertEqual(len(result), 27)

  def test_wordpiece_tokenize(self):
    """Test wordpiece tokenize function"""
    text = "This Python interpreter is in a conda environment, but the environment has not been activated.  Libraries may fail to load.  To activate this environment"
    result = self.WordpieceT.tokenize(text)
    self.assertEqual(len(result), 32)

  def test_search(self):
    """Test wordpiece tokenize function"""
    text = "This Python interpreter is in a conda environment, but the environment has not been activated.  Libraries may fail to load.  To activate this environment"
    instances, list_instances = search_word(text, 'this', tokenize = True, width = 3)
    self.assertEqual(instances, 2)
  
if __name__ == '__main__':
  unittest.main()



import re

from abc import ABCMeta, abstractmethod

class StemmerI(metaclass=ABCMeta):
    """
    A processing interface for removing morphological affixes from
    words.  This process is known as stemming.
    """

    @abstractmethod
    def stem(self, token):
        """
        Strip affixes from the token and return the stem.
        :param token: The token that should be stemmed.
        :type token: str
        """

class PorterStemmer(StemmerI):
    STEM_EXTENSION = "STEM_EXTENSION"

    def __init__(self, mode=STEM_EXTENSION):
        if mode not in (
            self.STEM_EXTENSION
        ):
            raise ValueError(
                "Mode must be one of PorterStemmer.STEM_EXTENSION, "
            )

        self.mode = mode

        if self.mode == self.STEM_EXTENSION:
            irregular_forms = {
                "sky": ["sky", "skies"],
                "die": ["dying"],
                "lie": ["lying"],
                "tie": ["tying"],
                "news": ["news"],
                "inning": ["innings", "inning"],
                "outing": ["outings", "outing"],
                "canning": ["cannings", "canning"],
                "howe": ["howe"],
                "proceed": ["proceed"],
                "exceed": ["exceed"],
                "succeed": ["succeed"],
            }

            self.pool = {}
            for key in irregular_forms:
                for val in irregular_forms[key]:
                    self.pool[val] = key

        self.vowels = frozenset(["a", "e", "i", "o", "u"])

    def _is_consonant(self, word, i):
    
        if word[i] in self.vowels:
            return False
        if word[i] == "y":
            if i == 0:
                return True
            else:
                return not self._is_consonant(word, i - 1)
        return True

    def _measure(self, stem):
        
        cv_sequence = ""

        # Construct a string of 'c's and 'v's representing whether each
        # character in `stem` is a consonant or a vowel.
        # e.g.  'architecture' becomes 'vcccvcvccvcv'
        for i in range(len(stem)):
            if self._is_consonant(stem, i):
                cv_sequence += "c"
            else:
                cv_sequence += "v"

        # Count the number of 'vc' occurrences, which is equivalent to
        # the number of 'VC' occurrences in Porter's reduced form in the
        # docstring above, which is in turn equivalent to `m`
        return cv_sequence.count("vc")

    def _has_positive_measure(self, stem):
        return self._measure(stem) > 0

    def _contains_vowel(self, stem):
        """Returns True if stem contains a vowel, else False"""
        for i in range(len(stem)):
            if not self._is_consonant(stem, i):
                return True
        return False

    def _ends_double_consonant(self, word):
        """Implements condition *d from the paper
        Returns True if word ends with a double consonant
        """
        return (
            len(word) >= 2
            and word[-1] == word[-2]
            and self._is_consonant(word, len(word) - 1)
        )

    def _ends_cvc(self, word):
        """Implements condition *o from the paper
        From the paper:
            *o  - the stem ends cvc, where the second c is not W, X or Y
                  (e.g. -WIL, -HOP).
        """
        return (
            len(word) >= 3
            and self._is_consonant(word, len(word) - 3)
            and not self._is_consonant(word, len(word) - 2)
            and self._is_consonant(word, len(word) - 1)
            and word[-1] not in ("w", "x", "y")
        ) or (
            self.mode == self.STEM_EXTENSION
            and len(word) == 2
            and not self._is_consonant(word, 0)
            and self._is_consonant(word, 1)
        )

    def _replace_suffix(self, word, suffix, replacement):
        """Replaces `suffix` of `word` with `replacement"""
        assert word.endswith(suffix), "Given word doesn't end with given suffix"
        if suffix == "":
            return word + replacement
        else:
            return word[: -len(suffix)] + replacement

    def _apply_rule_list(self, word, rules):
        """Suffix-removal rule to the word (in 3-tuples - suffix, word, condition)
        """
        for rule in rules:
            suffix, replacement, condition = rule
            if suffix == "*d" and self._ends_double_consonant(word):
                stem = word[:-2]
                if condition is None or condition(stem):
                    return stem + replacement
                else:
                    # Don't try any further rules
                    return word
            if word.endswith(suffix):
                stem = self._replace_suffix(word, suffix, "")
                if condition is None or condition(stem):
                    return stem + replacement
                else:
                    # Don't try any further rules
                    return word

        return word

    def _step1a(self, word):
        if self.mode == self.STEM_EXTENSION:
            if word.endswith("ies") and len(word) == 4:
                return self._replace_suffix(word, "ies", "ie")

        return self._apply_rule_list(
            word,
            [
                ("sses", "ss", None),  # SSES -> SS
                ("ies", "i", None),  # IES  -> I
                ("ss", "ss", None),  # SS   -> SS
                ("s", "", None),  # S    ->
            ],
        )

    def _step1b(self, word):
        """
            (m>0) EED -> EE                    feed      ->  feed
                                               agreed    ->  agree
            (*v*) ED  ->                       plastered ->  plaster
                                               bled      ->  bled
            (*v*) ING ->                       motoring  ->  motor
                                               sing      ->  sing
        
            AT -> ATE                       conflat(ed)  ->  conflate
            BL -> BLE                       troubl(ed)   ->  trouble
            IZ -> IZE                       siz(ed)      ->  size
            (*d and not (*L or *S or *Z))
               -> single letter
                                            hopp(ing)    ->  hop
                                            tann(ed)     ->  tan
            (m=1 and *o) -> E               fail(ing)    ->  fail
                                            fil(ing)     ->  file
        The rule to map to a single letter causes the removal of one of
        the double letter pair. The -E is put back on -AT, -BL and -IZ,
        so that the suffixes -ATE, -BLE and -IZE can be recognised
        later. This E may be removed in step 4.
        """
        if self.mode == self.STEM_EXTENSION:
            if word.endswith("ied"):
                if len(word) == 4:
                    return self._replace_suffix(word, "ied", "ie")
                else:
                    return self._replace_suffix(word, "ied", "i")

        # (m>0) EED -> EE
        if word.endswith("eed"):
            stem = self._replace_suffix(word, "eed", "")
            if self._measure(stem) > 0:
                return stem + "ee"
            else:
                return word

        rule_2_or_3_succeeded = False

        for suffix in ["ed", "ing"]:
            if word.endswith(suffix):
                intermediate_stem = self._replace_suffix(word, suffix, "")
                if self._contains_vowel(intermediate_stem):
                    rule_2_or_3_succeeded = True
                    break

        if not rule_2_or_3_succeeded:
            return word

        return self._apply_rule_list(
            intermediate_stem,
            [
                ("at", "ate", None),  # AT -> ATE
                ("bl", "ble", None),  # BL -> BLE
                ("iz", "ize", None),  # IZ -> IZE
                # (*d and not (*L or *S or *Z))
                # -> single letter
                (
                    "*d",
                    intermediate_stem[-1],
                    lambda stem: intermediate_stem[-1] not in ("l", "s", "z"),
                ),
                # (m=1 and *o) -> E
                (
                    "",
                    "e",
                    lambda stem: (self._measure(stem) == 1 and self._ends_cvc(stem)),
                ),
            ],
        )

    def _step1c(self, word):
        """
            (*v*) Y -> I                    happy        ->  happi
                                            sky          ->  sky
        """

        def nltk_condition(stem):
            """
            y->i is only done when y is preceded by a consonant,
            but not if the stem is only a single consonant, i.e.
               (*c and not c) Y -> I
            So 'happy' -> 'happi', but
               'enjoy' -> 'enjoy'  etc
            """
            return len(stem) > 1 and self._is_consonant(stem, len(stem) - 1)

        def original_condition(stem):
            return self._contains_vowel(stem)

        return self._apply_rule_list(
            word,
            [("y", "i",
                    nltk_condition
                    if self.mode == self.STEM_EXTENSION
                    else original_condition,
                )
            ],
        )

    def _step2(self, word):
        """
            (m>0) ATIONAL ->  ATE       relational     ->  relate
            (m>0) TIONAL  ->  TION      conditional    ->  condition
            (m>0) OUSNESS ->  OUS       callousness    ->  callous
            (m>0) ALITI   ->  AL        formaliti      ->  formal
            (m>0) IVITI   ->  IVE       sensitiviti    ->  sensitive
            (m>0) BILITI  ->  BLE       sensibiliti    ->  sensible
        """

        if self.mode == self.STEM_EXTENSION:
            # Instead of applying the ALLI -> AL rule after '(a)bli' per
            # the published algorithm, instead we apply it first, and,
            # if it succeeds, run the result through step2 again.
            if word.endswith("alli") and self._has_positive_measure(
                self._replace_suffix(word, "alli", "")
            ):
                return self._step2(self._replace_suffix(word, "alli", "al"))

        bli_rule = ("bli", "ble", self._has_positive_measure)
        abli_rule = ("abli", "able", self._has_positive_measure)

        rules = [
            ("ational", "ate", self._has_positive_measure),
            ("tional", "tion", self._has_positive_measure),
            ("enci", "ence", self._has_positive_measure),
            ("anci", "ance", self._has_positive_measure),
            ("izer", "ize", self._has_positive_measure),
            ("alli", "al", self._has_positive_measure),
            ("entli", "ent", self._has_positive_measure),
            ("eli", "e", self._has_positive_measure),
            ("ousli", "ous", self._has_positive_measure),
            ("ization", "ize", self._has_positive_measure),
            ("ation", "ate", self._has_positive_measure),
            ("ator", "ate", self._has_positive_measure),
            ("alism", "al", self._has_positive_measure),
            ("iveness", "ive", self._has_positive_measure),
            ("fulness", "ful", self._has_positive_measure),
            ("ousness", "ous", self._has_positive_measure),
            ("aliti", "al", self._has_positive_measure),
            ("iviti", "ive", self._has_positive_measure),
            ("biliti", "ble", self._has_positive_measure),
        ]

        if self.mode == self.STEM_EXTENSION:
            rules.append(("fulli", "ful", self._has_positive_measure))

            # The 'l' of the 'logi' -> 'log' rule is put with the stem,
            # so that short stems like 'geo' 'theo' etc work like
            # 'archaeo' 'philo' etc.
            rules.append(
                ("logi", "log", lambda stem: self._has_positive_measure(word[:-3]))
            )
        return self._apply_rule_list(word, rules)

    def _step3(self, word):
        """
            (m>0) ICATE ->  IC              triplicate     ->  triplic
            (m>0) ATIVE ->                  formative      ->  form
            (m>0) NESS  ->                  goodness       ->  good
        """
        return self._apply_rule_list(
            word,
            [
                ("icate", "ic", self._has_positive_measure),
                ("ative", "", self._has_positive_measure),
                ("alize", "al", self._has_positive_measure),
                ("iciti", "ic", self._has_positive_measure),
                ("ical", "ic", self._has_positive_measure),
                ("ful", "", self._has_positive_measure),
                ("ness", "", self._has_positive_measure),
            ],
        )

    def _step4(self, word):
        """
            (m>1) AL    ->                  revival        ->  reviv
            (m>1) ANCE  ->                  allowance      ->  allow
            (m>1) ENCE  ->                  inference      ->  infer
            (m>1) ER    ->                  airliner       ->  airlin
            (m>1) IC    ->                  gyroscopic     ->  gyroscop
            (m>1) ABLE  ->                  adjustable     ->  adjust
            (m>1) IBLE  ->                  defensible     ->  defens
            (m>1) ANT   ->                  irritant       ->  irrit
            (m>1) EMENT ->                  replacement    ->  replac
            (m>1) MENT  ->                  adjustment     ->  adjust
            (m>1) ENT   ->                  dependent      ->  depend
            (m>1 and (*S or *T)) ION ->     adoption       ->  adopt
            (m>1) OU    ->                  homologou      ->  homolog
            (m>1) ISM   ->                  communism      ->  commun
            (m>1) ATE   ->                  activate       ->  activ
            (m>1) ITI   ->                  angulariti     ->  angular
            (m>1) OUS   ->                  homologous     ->  homolog
            (m>1) IVE   ->                  effective      ->  effect
            (m>1) IZE   ->                  bowdlerize     ->  bowdler
        The suffixes are now removed. All that remains is a little
        tidying up.
        """
        measure_gt_1 = lambda stem: self._measure(stem) > 1

        return self._apply_rule_list(
            word,
            [
                ("al", "", measure_gt_1),
                ("ance", "", measure_gt_1),
                ("ence", "", measure_gt_1),
                ("er", "", measure_gt_1),
                ("ic", "", measure_gt_1),
                ("able", "", measure_gt_1),
                ("ible", "", measure_gt_1),
                ("ant", "", measure_gt_1),
                ("ement", "", measure_gt_1),
                ("ment", "", measure_gt_1),
                ("ent", "", measure_gt_1),
                # (m>1 and (*S or *T)) ION ->
                (
                    "ion",
                    "",
                    lambda stem: self._measure(stem) > 1 and stem[-1] in ("s", "t"),
                ),
                ("ou", "", measure_gt_1),
                ("ism", "", measure_gt_1),
                ("ate", "", measure_gt_1),
                ("iti", "", measure_gt_1),
                ("ous", "", measure_gt_1),
                ("ive", "", measure_gt_1),
                ("ize", "", measure_gt_1),
            ],
        )

    def _step5a(self, word):
        """Implements Step 5a from "An algorithm for suffix stripping"
        From the paper:
        Step 5a
            (m>1) E     ->                  probate        ->  probat
                                            rate           ->  rate
            (m=1 and not *o) E ->           cease          ->  ceas
        """
        if word.endswith("e"):
            stem = self._replace_suffix(word, "e", "")
            if self._measure(stem) > 1:
                return stem
            if self._measure(stem) == 1 and not self._ends_cvc(stem):
                return stem
        return word

    def _step5b(self, word):
        """Implements Step 5a from "An algorithm for suffix stripping"
        From the paper:
        Step 5b
            (m > 1 and *d and *L) -> single letter
                                    controll       ->  control
                                    roll           ->  roll
        """
        return self._apply_rule_list(
            word, [("ll", "l", lambda stem: self._measure(word[:-1]) > 1)]
        )

    def stem(self, sentence, to_lowercase=True):
        """
        :param to_lowercase: if `to_lowercase=True` the word always lowercase
        """
        words = sentence.split()
        stemmed_sentence = []
        for word in words:
            stem = word.lower() if to_lowercase else word

            if self.mode == self.STEM_EXTENSION and word in self.pool:
                return self.pool[stem]

            if len(word) <= 2:
                stemmed_sentence.append(stem)

            else:
                stem = self._step1a(stem)
                stem = self._step1b(stem)
                stem = self._step1c(stem)
                stem = self._step2(stem)
                stem = self._step3(stem)
                stem = self._step4(stem)
                stem = self._step5a(stem)
                stem = self._step5b(stem)

                stemmed_sentence.append(stem)
    
        str_stemmed_sentence = " ".join(stemmed_sentence)

        return str_stemmed_sentence

    def __repr__(self):
        return "<PorterStemmer>"



from pathlib import Path
import os
from tqdm import tqdm
import requests
from cmtt.preprocessing.sentence_piece_tokenizer.config import LanguageCodes, LMConfigs
from cmtt.data.downloader import download_file_from_google_drive

all_language_codes = LanguageCodes()

path = Path(__file__).parent


def download_file(url, dest, fname):
    if (dest/f'{fname}').exists(): return False
    os.makedirs(dest, exist_ok=True)
    print('Downloading Tokenizer Model...')
    
    response = requests.get(url, stream=True)
    total_size_in_bytes= int(response.headers.get('content-length', 0))
    block_size = 1024 #1 Kibibyte
    progress_bar = tqdm(total=total_size_in_bytes, unit='iB', unit_scale=True)
    with open(dest/f'{fname}', 'wb') as f:
        for data in response.iter_content(block_size):
            progress_bar.update(len(data))
            f.write(data)
    progress_bar.close()
    if total_size_in_bytes != 0 and progress_bar.n != total_size_in_bytes:
        print("ERROR, something went wrong")
        return False

    return True


def setup_language(language_code: str):
    lmconfig = LMConfigs(language_code)
    config = lmconfig.get_config()

    dest = path/'model'/f'{language_code}'
    fname = config["tokenizer_model_file_name"]
    
    if (dest/f'{fname}').exists(): 
        return False

    os.makedirs(dest, exist_ok=True)

    print('Downloading Tokenizer Model...')
    download_file_from_google_drive(config['tokenizer_model_id'], str(dest)+"/"+str(fname), str(fname),"")
    print('Download complete!')

    return True


def verify_tokenizer_model(language_code: str):
    lmconfig = LMConfigs(language_code)
    config = lmconfig.get_config()
    if (path/'models'/f'{language_code}'/f'{config["tokenizer_model_file_name"]}').exists():
        return True
    else:
        return False



from readline import set_startup_hook
from fastai.text import *
from cmtt.preprocessing.sentence_piece_tokenizer.config import LanguageCodes
from cmtt.preprocessing.sentence_piece_tokenizer.download_assets import setup_language, verify_tokenizer_model
import sentencepiece as spm
from pathlib import Path
from typing import List

all_language_codes = LanguageCodes().get_all_language_codes()

def check_input_language(language_code: str):
    try:
        if language_code not in all_language_codes:
            raise ValueError()
    except ValueError as err:
        print("Error: Language Not Supported")
        # return -1

    if not verify_tokenizer_model(language_code):
        return -1

    return 1


path = Path(__file__).parent

class Sentencepiece_tokenizer():
    def __init__(self, lang: str):
        self.lang = lang
        if (check_input_language(self.lang)==-1):
            print("Tokenizer model not downloaded. Starting Download...")
            setup_language(self.lang)
        self.sp = spm.SentencePieceProcessor()
        model_path = path/f'models/{lang}/tokenizer.model'
        self.sp.Load(str(model_path))

    def tokenize(self, t: str) -> List[str]:
        return self.sp.EncodeAsPieces(t)

    def numericalize(self, t: str) -> List[int]:
        return self.sp.EncodeAsIds(t)

    def textify(self, ids: List[int]) -> str:
        return (''.join([self.sp.IdToPiece(id).replace('▁', ' ') for id in ids])).strip()

    def remove_foreign_tokens(self, t: str):
        local_pieces = []
        for i in self.sp.EncodeAsIds(t):
            local_pieces.append(self.sp.IdToPiece(i))
        return local_pieces

    def detokenize(self, tokens):
        if type(tokens) is list:
            text = ""
            for i in tokens:
                if "▁" in i:
                    text += " " + i[1:]
                else:
                    text += i
            return text.strip()    
        else:
            raise TypeError('Tokens needs to be of type list. Expected type list but got type ' + str(type(tokens)))




from fastai.text import * 
from pathlib import *

import sentencepiece as spm

# Path to the directory containing the training files
path = pathlib.Path(r'')

p = path.glob('**/*')
files = [x for x in p if x.is_file()]
files = [str(file) for file in files]
flist = ','.join(files)

custom_symbols = [text.transform.FLD, 
                text.transform.TK_MAJ,
                text.transform.TK_UP,
                text.transform.TK_REP,
                text.transform.TK_WREP]

str_specialcases = ",".join(custom_symbols)

# Trained Models uploaded to the CMTT cloud
spm.SentencePieceTrainer.Train(f'--input={flist} --model_prefix=hinDev_engRom --vocab_size=60000 --input_sentence_size=22500000 --unk_id=0 --bos_id=1 --eos_id=2 --pad_id=3 --unk_piece={text.transform.UNK} --bos_piece={text.transform.BOS} --eos_piece={text.transform.EOS} --pad_piece={text.transform.PAD} --user_defined_symbols={str_specialcases}')


from fastai.text import *
from googletrans import Translator
import re
from cmtt.preprocessing.stemmer.stemmer import *
from cmtt.LID import *

words_dict  = { "तैराक":"तैर",
                "चालाक":"चाल",
                "कूलाक":"कूल",
                "बेलन":"बेल",
                "मिलाप":"मिल",
                "चुपचाप": "चुप",
                "निकास":"निकस",
                "लुकास":"लुक",
                }

suffixes = {
	    1: ["ो", "े", "ू", "ु", "ी", "ि", "ा"],  
            2: ["तृ","ान","ैत","ने","ाऊ","ाव","कर", "ाओ", "िए", "ाई", "ाए", "नी", "ना", "ते", "ीं", "ती",
                "ता", "ाँ", "ां", "ों", "ें","ीय", "ति","या", "पन", "पा","ित","ीन","लु","यत","वट","लू"],     
            3: ["ेरा","त्व","नीय","ौनी","ौवल","ौती","ौता","ापा","वास","हास","काल","पान","न्त","ौना","सार","पोश","नाक",
                "ियल","ैया", "ौटी","ावा","ाहट","िया","हार", "ाकर", "ाइए", "ाईं", "ाया", "ेगी", "वान", "बीन",
                "ेगा", "ोगी", "ोगे", "ाने", "ाना", "ाते", "ाती", "ाता", "तीं", "ाओं", "ाएं", "ुओं", "ुएं", "ुआं","कला","िमा","कार",
                "गार", "दान","खोर"],     
            4: ["ावास","कलाप","हारा","तव्य","वैया", "वाला", "ाएगी", "ाएगा", "ाओगी", "ाओगे", 
                "एंगी", "ेंगी", "एंगे", "ेंगे", "ूंगी", "ूंगा", "ातीं", "नाओं", "नाएं", "ताओं", "ताएं", "ियाँ", "ियों", "ियां",
                "त्वा","तव्य","कल्प","िष्ठ","जादा","क्कड़"],     
            5: ["ाएंगी", "ाएंगे", "ाऊंगी", "ाऊंगा", "ाइयाँ", "ाइयों", "ाइयां", "अक्कड़","तव्य:","निष्ठ"],
}

special_suffixes = ["र्", "ज्य","त्य"]
dict_special_suffixes = {"र्":"ृ",
                         "ज्य":"ज्",
                         "त्य":"त्"}

def hi_stem(word, clean=False,chars=None):
    if clean == True:
        word = clean_text(word, chars)
    
    ans = word
    bl = False
    
    if word in words_dict.keys():
        return words_dict[word]
    
    for L in 5, 4, 3, 2, 1:
        if len(word) > L + 1:
            for suf in suffixes[L]:
                if word.endswith(suf):
                    ans = word[:-L]
                    bl =True
        if bl == True:
            break
                    
    if bl == True:
        for suf in suffixes[1]:
            if ans.endswith(suf):
                # use case - गानेवाला
                ans = hi_stem(ans)
   
    for suf in special_suffixes:
        if ans.endswith(suf):
            l = len(suf)
            ans = ans[:-l]
            ans += dict_special_suffixes[suf]
 
    return ans

def clean_text(text, chars=None):
    if chars == None:        
        text = re.sub(r"[()\"#/@;:<>{}`+=~|!?,']", "", text)
    else:
        text = re.sub(r"[" +chars+ "()\"#/@;:<>{}`+=~|!?,']", "", text)
    return text

def hindi_stem(text):
    ans = ""
    for i in text.split(' '):
        ans += hi_stem(i)
        ans += " "

    return ans

def hien_stemmer(text):
  results = []
  
  lid = BiLSTM_LID()
  translator = Translator()
  en_stemmer = PorterStemmer()

  tokens_tags_list = lid.getLangTags(text)

  for i in tokens_tags_list:
    if(i[1] == 'EN'):
      results.append(en_stemmer.stem(i[0]))
    else:
      hi_token = translator.translate(i[0], src = 'hi', dest = 'hi').text
      if (bool(re.match(r"^[A-Za-z]", hi_token))):
        hi_token = translator.translate(i[0], src = 'hi', dest = 'hi')
      results.append(hindi_stem(hi_token))

  return results


import unicodedata
import os
import re

def convert_to_unicode(text):
  """Converts `text` to Unicode (if it's not already), assuming utf-8 input."""
  if isinstance(text, str):
    return text
  elif isinstance(text, bytes):
    return text.decode("utf-8", "ignore")
  else:
    raise ValueError("Unsupported string type: %s" % (type(text)))

##################################################
# Whitespace Tokenizer
##################################################
class WhitespaceTokenizer(object):
  def tokenize(self, text):
    text = text.strip()
    if not text:
      return []
    tokens = text.split()
    return tokens

  def detokenize(self, tokens):
    if type(tokens) is list:
      return ' '.join(tokens)
    else:
      raise TypeError('Tokens needs to be of type list. Expected type list but got type ' + str(type(tokens)))

##################################################
# Word Tokenizer
##################################################
class WordTokenizer(object):
  """Runs basic tokenization (punctuation splitting, lower casing, etc.)."""

  def __init__(self, do_lower_case=True):
    """Constructs a BasicTokenizer.
    Args:
      do_lower_case: Whether to lower case the input.
    """
    self.do_lower_case = do_lower_case

  def tokenize(self, text):
    """Tokenizes a piece of text."""
    text = convert_to_unicode(text)
    text = self._clean_text(text)

    WhitespaceT = WhitespaceTokenizer()
    orig_tokens = WhitespaceT.tokenize(text)
    split_tokens = []
    for token in orig_tokens:
      if self.do_lower_case:
        token = token.lower()
        token = self._run_strip_accents(token)
      split_tokens.extend(self._run_split_on_punc(token))

    output_tokens = WhitespaceT.tokenize(" ".join(split_tokens))
    return output_tokens

  def detokenize(self, tokens):
    if type(tokens) is list:
      text = ""
      for i in tokens:
        if len(i) == 1:
          if self._is_punctuation(i):
            text += i
          else:
            text += " " + i
        else:
          text += " " + i
      return text.strip()    
    else:
      raise TypeError('Tokens needs to be of type list. Expected type list but got type ' + str(type(tokens)))

  def _run_strip_accents(self, text):
    """Strips accents from a piece of text."""
    text = unicodedata.normalize("NFD", text)
    output = []
    for char in text:
      cat = unicodedata.category(char)
      if cat == "Mn":
        continue
      output.append(char)
    return "".join(output)

  def _run_split_on_punc(self, text):
    """Splits punctuation on a piece of text."""
    chars = list(text)
    i = 0
    start_new_word = True
    output = []
    while i < len(chars):
      char = chars[i]
      if self._is_punctuation(char):
        output.append([char])
        start_new_word = True
      else:
        if start_new_word:
          output.append([])
        start_new_word = False
        output[-1].append(char)
      i += 1

    return ["".join(x) for x in output]

  def _clean_text(self, text):
    """Performs invalid character removal and whitespace cleanup on text."""
    output = []
    for char in text:
      cp = ord(char)
      if cp == 0 or cp == 0xfffd or self._is_control(char):
        continue
      if self._is_whitespace(char):
        output.append(" ")
      else:
        output.append(char)
    return "".join(output)

  def _is_whitespace(self, char):
    """Checks whether `chars` is a whitespace character."""
    # \t, \n, and \r are technically contorl characters but we treat them
    # as whitespace since they are generally considered as such.
    if char == " " or char == "\t" or char == "\n" or char == "\r":
      return True
    cat = unicodedata.category(char)
    if cat == "Zs":
      return True
    return False

  def _is_control(self, char):
    """Checks whether `chars` is a control character."""
    # These are technically control characters but we count them as whitespace
    # characters.
    if char == "\t" or char == "\n" or char == "\r":
      return False
    cat = unicodedata.category(char)
    if cat in ("Cc", "Cf"):
      return True
    return False

  def _is_punctuation(self, char):
    """Checks whether `chars` is a punctuation character."""
    cp = ord(char)
    # We treat all non-letter/number ASCII as punctuation.
    # Characters such as "^", "$", and "`" are not in the Unicode
    # Punctuation class but we treat them as punctuation anyways, for
    # consistency.
    if ((cp >= 33 and cp <= 47) or (cp >= 58 and cp <= 64) or
        (cp >= 91 and cp <= 96) or (cp >= 123 and cp <= 126)):
      return True
    cat = unicodedata.category(char)
    if cat.startswith("P"):
      return True
    return False

##################################################
# Wordpiece Tokenizer
##################################################
def load_vocab_list():
  """Loads a vocabulary file into a dictionary."""
  vocab = []
  path = os.path.dirname(os.path.realpath(__file__))
  f = open(os.path.join(path, "vocab_2.txt"), 'r')
 
  with f as reader:
    while True:
      token = convert_to_unicode(reader.readline())
      if not token:
        break
      token = token.strip()
      vocab.append(token)
  return vocab 

class Wordpiece_tokenizer(object):
  def __init__(self, vocab = load_vocab_list()):
    self.vocab = vocab

  def encode_word(self, word):
    tokens = []
    while len(word) > 0:
      i = len(word)
      while i > 0 and word[:i].lower() not in self.vocab:
        i -= 1
      if i == 0:
        return ["[UNK]"]
      tokens.append(word[:i])
      word = word[i:]
      if len(word) > 0:
        word = f"##{word}"
    return tokens

  def tokenize(self, text):
    """
      Tokenizes a piece of text into its word pieces.
      For example:
        input = "unaffable"
        output = ["un", "##aff", "##able"]
      Args:
        text: A single token or whitespace separated tokens. This should have already been passed through `BasicTokenizer.
      Returns:
        A list of wordpiece tokens.
    """
    text = text.strip()
    text = re.findall(r"[\w]+|[^\s\w]", text)
    encoded_words = [self.encode_word(word) for word in text]
    return sum(encoded_words, [])

  def detokenize(self, tokens):
    if type(tokens) is list:
      text = ""
      for i in tokens:
        if len(i) == 1:
          if self._is_punctuation(i):
            text += i
          else:
            text += " " + i
        elif "##" in i:
          text += i[2:]
        else:
          text += " " + i
      return text.strip()    
    else:
      raise TypeError('Tokens needs to be of type list. Expected type list but got type ' + str(type(tokens)))

  def _is_punctuation(self, char):
    """Checks whether `chars` is a punctuation character."""
    cp = ord(char)
    # We treat all non-letter/number ASCII as punctuation.
    # Characters such as "^", "$", and "`" are not in the Unicode
    # Punctuation class but we treat them as punctuation anyways, for
    # consistency.
    if ((cp >= 33 and cp <= 47) or (cp >= 58 and cp <= 64) or
        (cp >= 91 and cp <= 96) or (cp >= 123 and cp <= 126)):
      return True
    cat = unicodedata.category(char)
    if cat.startswith("P"):
      return True
    return False

##################################################
# Devanagari Tokenizer
##################################################
class DevanagariTokenizer(object):
  def word_tokenize(self, text):
    text = text.strip()
    processed_text = ""
    for i in text:
      if self._is_punctuation(i):
        processed_text += " " + i
      else:
        processed_text += i
    
    tokens = processed_text.split(" ")
    return tokens        

  def character_tokenize(self, text):
    text = text.strip()
    if not text:
      return []
    tokens = re.findall(r"[\w]+|[^\s\w]", text)
    return tokens

  def word_detokenize(self, tokens):
    if type(tokens) is list:
      text = ""
      for i in tokens:
        if len(i) == 1:
          if self._is_punctuation(i):
            text += i
          else:
            text += " " + i
        else:
          text += " " + i
      return text.strip()   
    else:
      raise TypeError('Tokens needs to be of type list. Expected type list but got type ' + str(type(tokens)))

  def _is_punctuation(self, char):
    """Checks whether `chars` is a punctuation character."""
    cp = ord(char)
    # We treat all non-letter/number ASCII as punctuation.
    # Characters such as "^", "$", and "`" are not in the Unicode
    # Punctuation class but we treat them as punctuation anyways, for
    # consistency.
    if ((cp >= 33 and cp <= 47) or (cp >= 58 and cp <= 64) or
        (cp >= 91 and cp <= 96) or (cp >= 123 and cp <= 126)):
      return True
    cat = unicodedata.category(char)
    if cat.startswith("P"):
      return True
    return False


# Code Mixed Text Toolkit
from cmtt.preprocessing.tokenizer import *
from cmtt.preprocessing.search import *
from cmtt.preprocessing.sentence_piece_tokenizer import *
from cmtt.preprocessing.stemmer import *



from pathlib import Path
import torch
import torch.nn as nn
import os
import dill
from cmtt.preprocessing import *
from cmtt.data.downloader import download_file_from_google_drive

# Training Colab
# https://colab.research.google.com/drive/1mPJCkXroReuPTb3XP0uTByy1TI5vTBc4#scrollTo=JX9dYDOhpwmy

model_files = [
    {   
        'id': "1uqOy7EgoClJ1ShPaqS1blCJGfLD14_Wy",
        'fname': "TEXT.Field", 
    },
    {   
        'id': "1N0uQ7KOwtemXO7t8XbR1_iZAkACktbgE",
        'fname': "LABEL.Field", 
    },
    {   
        'id': "1WwjZsMzVbRbEqBrPED4ZqR81jXzLRv_6",
        'fname': "lid_model.pt", 
    }
]

path = Path(__file__).parent

device = torch.device('cuda' if torch.cuda.is_available() else 'cpu')

class BiLSTM_MODEL(nn.Module):
    def __init__(self, 
                 input_dim, 
                 embedding_dim, 
                 hidden_dim, 
                 output_dim, 
                 n_layers, 
                 bidirectional, 
                 dropout, 
                 pad_idx):
        
        super().__init__()
        
        self.embedding = nn.Embedding(input_dim, embedding_dim, padding_idx = pad_idx)
        
        self.lstm = nn.LSTM(embedding_dim, 
                            hidden_dim, 
                            num_layers = n_layers, 
                            bidirectional = bidirectional,
                            dropout = dropout if n_layers > 1 else 0)
        
        self.fc = nn.Linear(hidden_dim * 2 if bidirectional else hidden_dim, output_dim)
        
        self.dropout = nn.Dropout(dropout)
        
    def forward(self, text):
        embedded = self.dropout(self.embedding(text))
        outputs, (hidden, cell) = self.lstm(embedded)
        predictions = self.fc(self.dropout(outputs))
        return predictions


class BiLSTM_LID():

    def __init__(self):

        dest = path/'models/'
        if not (dest).exists(): 
            os.makedirs(dest, exist_ok=True)
            print("Downloading model files..")
            for i in model_files:
                download_file_from_google_drive(i['id'], str(dest)+"/"+str(i['fname']), i['fname'], "")
            print("Files downloaded")

        with open(os.path.join(path/'models', "TEXT.Field"),"rb")as f:
            self.TEXT=dill.load(f)

        with open(os.path.join(path/'models', "LABEL.Field"),"rb")as f:
            self.LABEL=dill.load(f)

        INPUT_DIM = len(self.TEXT.vocab)
        EMBEDDING_DIM = 100
        HIDDEN_DIM = 128
        OUTPUT_DIM = len(self.LABEL.vocab)
        N_LAYERS = 2
        BIDIRECTIONAL = True
        DROPOUT = 0.25
        PAD_IDX = self.TEXT.vocab.stoi[self.TEXT.pad_token]

        self.model = BiLSTM_MODEL(INPUT_DIM, 
                                EMBEDDING_DIM, 
                                HIDDEN_DIM, 
                                OUTPUT_DIM, 
                                N_LAYERS, 
                                BIDIRECTIONAL, 
                                DROPOUT, 
                                PAD_IDX)

        pretrained_embeddings = self.TEXT.vocab.vectors

        self.model.embedding.weight.data[PAD_IDX] = torch.zeros(EMBEDDING_DIM)

        self.model = self.model.to(device)

        self.model.load_state_dict(torch.load(os.path.join(path/'models', "lid_model.pt"), map_location=torch.device('cpu')))

    def tag_sentence(self, device, sentence, text_field, tag_field):
        self.model.eval()
        if isinstance(sentence, str):
            # nlp = spacy.load('en_core_web_sm')
            WordT = WordTokenizer()
            tokenized = WordT.tokenize(sentence)
            tokens = []
            for i in tokenized:
                if len(i) != 1 or (len(i) == 1 and not WordT._is_punctuation(i)): 
                    tokens.append(i)             

            # tokens = [token.text for token in nlp(sentence)]
        else:
            tokens = [token for token in sentence]

        if text_field.lower:
            tokens = [t.lower() for t in tokens]
            
        numericalized_tokens = [text_field.vocab.stoi[t] for t in tokens]
        unk_idx = text_field.vocab.stoi[text_field.unk_token]
        unks = [t for t, n in zip(tokens, numericalized_tokens) if n == unk_idx]
        token_tensor = torch.LongTensor(numericalized_tokens)
        token_tensor = token_tensor.unsqueeze(-1).to(device)
        predictions = self.model(token_tensor)
        top_predictions = predictions.argmax(-1)
        predicted_tags = [tag_field.vocab.itos[t.item()] for t in top_predictions]
        return tokens, predicted_tags, unks

    def getLangTags(self, sentence):
        tokens, predicted_tags, unks = self.tag_sentence(device, sentence, self.TEXT, self.LABEL)
        tuple_list = []
        for i in range(len(tokens)):
            tuple_list.append((tokens[i], predicted_tags[i]))

        return tuple_list

    def getUnks(self, sentence):
        tokens, predicted_tags, unks = self.tag_sentence(device, sentence, self.TEXT, self.LABEL)
        return unks




# Code Mixed Text Toolkit
# Download dataset given name (list) or url
# Save into folder corpora in C file
# Show progress in command line
# Download finished

import pickle
import os
import json
import pandas as pd
import urllib.request
import requests
from urllib.request import urlopen
from tqdm import tqdm
import zipfile
import platform

def download_file(url, destination, name, format):
  """
    Downloads a file from a url into the destination folder
    :param url: the source url of the dataset
    :type url: str
    :param destination: the destination path of the dataset to be downloaded
    :type destination: str
    :param name: name of the dataset at the destination folder
    :type name: str
    :param format: format of the dataset to be downloaded
    :type format: str
  """
  fname = destination
  response = requests.get(url, stream=True)
  total_size_in_bytes= int(response.headers.get('content-length', 0))
  block_size = 1024 #1 Kibibyte
  progress_bar = tqdm(total=total_size_in_bytes, unit='iB', unit_scale=True)
  with open(fname, 'wb') as file:
    for data in response.iter_content(block_size):
      progress_bar.update(len(data))
      file.write(data)
  progress_bar.close()
  if total_size_in_bytes != 0 and progress_bar.n != total_size_in_bytes:
    print("ERROR, something went wrong")
  else:
    print("\nDataset file successfully downloaded into path: " + destination)
    if (format == "zip"):
      print("Downloaded file is a zip file. Extracting zip file..")
      os.makedirs(os.path.join(os.path.dirname(destination), name))
      destination_after_extraction = os.path.join(os.path.dirname(destination), name)
      with zipfile.ZipFile(destination, 'r') as zip_ref:
          zip_ref.extractall(destination_after_extraction)
      os.remove(destination)
      print("Successfully extracted downloaded file into path: " + destination_after_extraction)
      return destination_after_extraction

    return destination

def download_file_from_google_drive(id, destination, name, format):
  """
    General method for downloading a file from Google Drive.
    Doesn't require using API or having credentials
    :param id: the source google drive file id of the dataset
    :type id: str
    :param destination: the destination path of the dataset to be downloaded
    :type destination: str
    :param name: name of the dataset at the destination folder
    :type name: str
    :param format: format of the dataset to be downloaded
    :type format: str
  """

  URL = "https://docs.google.com/uc?export=download"

  session = requests.Session()

  response = session.get(URL, params = { 'id' : id }, stream = True)
  token = get_confirm_token(response)

  if token:
    params = { 'id' : id, 'confirm' : token }
    response = session.get(URL, params = params, stream = True)

  return save_response_content(response, destination, name, format)    

def get_confirm_token(response):
  """
    Part of keep-alive method for downloading large files from Google Drive
    Discards packets of data that aren't the actual file
    :param response: session-based google query
    :return: either datapacket or discard unneeded data
  """

  # Download warning appears while trying to download large files from google drive
  for key, value in response.cookies.items():
    if key.startswith('download_warning'):
      return value

  return None

def save_response_content(response, destination, name, format):
  """
    Saves and writes the response data into the destination folder
    :param response: response after executing a get request
    :type response: requests.Response object
    :param destination: the destination path of the dataset to be downloaded
    :type destination: str
    :param name: name of the dataset at the destination folder
    :type name: str
    :param format: format of the dataset to be downloaded
    :type format: str
  """

  CHUNK_SIZE = 1024
  total_size_in_bytes= int(response.headers.get('content-length', 0))
  progress_bar = tqdm(total=total_size_in_bytes, unit='iB', unit_scale=True)
  with open(destination, "wb") as f:
    for chunk in response.iter_content(CHUNK_SIZE):
      if chunk:
        progress_bar.update(len(chunk))
        f.write(chunk)
  progress_bar.close()
  if total_size_in_bytes != 0 and progress_bar.n != total_size_in_bytes:
    print("ERROR, something went wrong")
  else:
    print("\nDataset file successfully downloaded into path: " + destination)
    if (format == "zip"):
      print("Downloaded file is a zip file. Extracting zip file..")
      os.makedirs(os.path.join(os.path.dirname(destination), name))
      destination_after_extraction = os.path.join(os.path.dirname(destination), name)
      with zipfile.ZipFile(destination, 'r') as zip_ref:
        zip_ref.extractall(os.path.join(os.path.dirname(destination), name))
      os.remove(destination)
      print("Successfully extracted downloaded file into path: " + destination_after_extraction)
      return destination_after_extraction

    return destination
  

def download_cmtt_datasets(name_list):
  """
    Downloads an inbuilt cmtt dataset present in data.json into the user profile directory
    :param name_list: list of cmtt datasets to be downloaded
    :type name_list: list
  """

  if(len(name_list) == 0):
    print("Empty list provided")
    return

  path = os.path.dirname(os.path.realpath(__file__))
  f = open(os.path.join(path, "data.json"))
  data = json.load(f)

  flag = 0

  path_lists = []
  for name in name_list:
    for i in data['datasets']:
      if i['name'] == name:
        url = i['url']
        file_id = i['id']
        source = i['source']
        format = i['format']
        language = i['language']
        flag = 1
        break
    if(flag == 0):
      print("No dataset with the name {} exists".format(name))
      continue
    else:
      flag = 0

    user_root = os.path.expanduser("~")
    newpath = os.path.join(user_root, "cmtt")

    if not os.path.exists(newpath):
      os.makedirs(newpath)
    
    if not os.path.exists(os.path.join(newpath, language)):
      os.makedirs(os.path.join(newpath, language))

    newpath  = os.path.join(newpath, language)
    if os.path.exists(os.path.join(newpath, name+"."+format)) or os.path.isdir(os.path.join(newpath, name)):
      print("Dataset " + name + " already exists at path: "  + newpath)
      if format == "zip":
        path = os.path.join(newpath, name)
      else:
        path = os.path.join(newpath, name+"."+format)
      path_lists.append(path)
    else:
      print("Dataset " + name + " download starting...")
      try:
        if source == "gdrive":
          path = download_file_from_google_drive(file_id, os.path.join(newpath, name+"."+format), name, format)
        else:
          path = download_file(url, os.path.join(newpath, name+"."+format), name, format)
        path_lists.append(path)
      except:
        print("Download failed while downloading " + name)

  f.close()
  return path_lists
            

def download_dataset_url(url, fileName="defaultFileName"):
  """
    Downloads a dataset from a url into 'C:\cmtt'
    :param url: the source url of the dataset
    :type url: str
    :param destination: the filename of the dataset in destination after download
    :type destination: str
  """

  url_parts = url.split(".")
  format = url_parts[-1]
  if format == "gz":
    format = url_parts[-2]

  # newpath = r'C:\cmtt'
  newpath = os.path.join(os.getcwd(), "datasets")
  if not os.path.exists(newpath):
    os.makedirs(newpath)
  if os.path.exists(os.path.join(newpath, fileName+"."+format)) or os.path.isdir(os.path.join(newpath, fileName)):
    print("A dataset with filename " + fileName + " already exists at path: "  + newpath)
    if format == "zip":
      path = os.path.join(newpath, fileName)
    else:
      path = os.path.join(newpath, fileName+"."+format)
  else:
    print("Dataset " + fileName + " download starting...")
    try:
      path = download_file(url, os.path.join(newpath, fileName+"."+format), fileName, format)
    except:
      print("Download failed while downloading " + fileName) 
          
  return path


class LanguageCodes:
    hindi = 'hi'
    hinglish = 'hi-en'
    english = 'en'
    hinDev_engRom = 'hinDev_engRom'

    def get_all_language_codes(self):
        return [self.hindi, self.hinglish, self.english, self.hinDev_engRom]


class LMConfigs:
    all_language_codes = LanguageCodes()
    tokenizer_model_file_id = {
        all_language_codes.hindi: '1rtHpZjR65JyF6Km43Me_07uJDsDGmJUq',
        all_language_codes.hinglish: '1rKQufGrxZ881yHry7B4vPGJlC83wyjTT',
        all_language_codes.english: '1Cz1LSYjebfR11cNxjUyeBbEfvFF1znJ8',
        all_language_codes.hinDev_engRom: '1EPNWOPzH4PVWQlg6KgnVbQ9INZ7DN7eX',
        # all_language_codes.hindi: 'https://www.dropbox.com/s/5p19jcd4jlzf6r1/hindi_tokenizer.model?dl=0',
        # all_language_codes.hinglish: 'https://www.dropbox.com/s/j5itqvse5u77urs/hinglish_tokenizer.model?dl=0',
        # all_language_codes.english: 'https://www.dropbox.com/s/kpks31yovdhc20l/english_tokenizer.model?dl=0',
    }

    def __init__(self, language_code: str):
        self.language_code = language_code

    def get_config(self):
        return {
            'tokenizer_model_id': self.tokenizer_model_file_id[self.language_code],
            'tokenizer_model_file_name': 'tokenizer.model'
        }


# Always prefer setuptools over distutils
from setuptools import setup, find_packages

# To use a consistent encoding
from codecs import open
from os import path

# The directory containing this file
HERE = path.abspath(path.dirname(__file__))

# Get the long description from the README file
with open(path.join(HERE, 'README.md'), encoding='utf-8') as f:
  long_description = f.read()

# This call to setup() does all the work
setup(
  name="cmtt",
  version="0.7.5",
  description="A library for processing Code Mixed Text. Still in development!",
  long_description_content_type="text/markdown",
  long_description=long_description,
  url="https://cmtt.readthedocs.io/",
  author="Reuben Devanesan",
  author_email="reubendevanesan@gmail.com",
  license="MIT",
  classifiers=[
    "Intended Audience :: Developers",
    "License :: OSI Approved :: MIT License",
    "Programming Language :: Python",
    "Programming Language :: Python :: 3",
    "Programming Language :: Python :: 3.9",
    "Programming Language :: Python :: 3.10",
    "Operating System :: OS Independent"
  ],
  dependency_links=[
    'http://download.pytorch.org/whl/cpu/torch-1.0.0-cp36-cp36m-linux_x86_64.whl'
  ],
  # packages=["cmtt", "cmtt/data", "cmtt/preprocessing"],
  packages=find_packages(),
  include_package_data=True,
  data_files=[('cmtt/data', ['cmtt/data/data.json']), ('cmtt/preprocessing/tokenizer', ['cmtt/preprocessing/tokenizer/vocab.txt', 'cmtt/preprocessing/tokenizer/vocab_2.txt'])],
  install_requires=[
    "numpy", 
    "pandas", 
    "requests", 
    "tqdm",
    'fastai==1.0.57',
    "sentencepiece",
    "torch==1.8.0",
    "dill",
    "torchtext==0.9.0",
    "googletrans"
  ]
)


# Code Mixed Text Toolkit (CMTT)

"""
Code Mixed Text Toolkit (CMTT) is a wrapper library that makes code-mixed text processing more efficient than ever.
"""

from cmtt.data import *
from cmtt.preprocessing import *
from cmtt.LID import *


# Code Mixed Text Toolkit
"""
Functions to load local and online resource files, list and download CMTT datasets.
"""

import pickle
import json
import pandas as pd
from urllib.request import urlopen
import os

from cmtt.data.downloader import download_dataset_url, download_cmtt_datasets

AUTO_FORMATS = {
  "pickle": "pickle",
  "json": "json",
  "txt": "text",
  "text": "text",
  "csv": "csv",
  "conll": "conll"
}

FORMATS = {
  "pickle": "A serialized python object, stored using the pickle module.",
  "json": "A serialized python object, stored using the json module.",
  "text": "The raw (unicode string) contents of a file.",
  "csv": "A serialized python object, stored using the pandas module.",
  "conll": ""
}

def load_url(
  resource_url,
  format="auto",
  encoding=None):

  '''suported resource formats:
    -"txt"
    -"csv"
    -"pickle"
    -"json"
    -"raw"
  '''
  
  # Determine the format of the resource.
  if format == "auto":
    resource_url_parts = resource_url.split(".")
    ext = resource_url_parts[-1]
    if ext == "gz":
      ext = resource_url_parts[-2]
    format = AUTO_FORMATS.get(ext)
    if format is None:
      raise ValueError(
        "Could not determine format for %s based "
        'on its file\nextension; use the "format" '
        "argument to specify the format explicitly." % resource_url
      )
  
  if format not in FORMATS:
    raise ValueError(f"Unknown format type: {format}!")

  # Load the resource using URL.
  opened_resource = urlopen(resource_url)

  if format == "raw":
    resource_val = opened_resource.read()
  elif format == "pickle":
    resource_val = pickle.load(opened_resource)
  elif format == "json":
    resource_val = json.load(opened_resource)
  elif format == "csv":
    resource_val = pd.read_csv(opened_resource)
  else:
    # The resource is a text format.
    binary_data = opened_resource.read()
    if encoding is not None:
      string_data = binary_data.decode(encoding)
    else:
      try:
        string_data = binary_data.decode("utf-8")
      except UnicodeDecodeError:
        string_data = binary_data.decode("latin-1")
    if format == "text":
      resource_val = string_data

  opened_resource.close()
  return resource_val

def load_local(
  resource_path,
  format="auto",
  encoding=None):

  '''suported resource formats:
    -"txt"
    -"csv"
    -"pickle"
    -"json"
    -"raw"
    -"conll"
  '''
  
  # Determine the format of the resource.
  if format == "auto":
    resource_path_parts = resource_path.split(".")
    ext = resource_path_parts[-1]
    if ext == "gz":
      ext = resource_path_parts[-2]
    format = AUTO_FORMATS.get(ext)
    if format is None:
      raise ValueError(
        "Could not determine format for %s based "
        'on its file\nextension; use the "format" '
        "argument to specify the format explicitly." % resource_path
      )
  
  if format not in FORMATS:
    raise ValueError(f"Unknown format type: {format}!")

  # Load the resource using URL.
  opened_resource = open(resource_path, 'rb')

  if format == "raw":
    resource_val = opened_resource.read()
  elif format == "pickle":
    resource_val = pickle.load(opened_resource)
  elif format == "json":
    resource_val = json.load(opened_resource)
  elif format == "csv":
    resource_val = pd.read_csv(opened_resource)
  else:
    # The resource is a text format.
    binary_data = opened_resource.read()
    if encoding is not None:
      string_data = binary_data.decode(encoding)
    else:
      try:
        string_data = binary_data.decode("utf-8")
      except UnicodeDecodeError:
        string_data = binary_data.decode("latin-1")
    if format == "text":
      resource_val = string_data
    if format == "conll":
      resource_val = string_data
  opened_resource.close()
  return resource_val


def list_dataset_keys():
  """
    Returns the list of key prperties of the datasets provided by the cmtt library
    :return: list of keys
    :rtype: list
  """

  path = os.path.dirname(os.path.realpath(__file__))
  f = open(os.path.join(path, "data.json"))
  data = json.load(f)
  f.close()
  return list(data['datasets'][0].keys())

def list_cmtt_datasets(search_key="all", search_term = "", isPrint=False):
  """
    Returns the list the datasets provided by the cmtt library.
    :param search_key: dataset property in which the search_term is to searched. 
    :type key: str
    :param search_term: term to be search in the list of datasets
    :type key: str
    :param print: turn on and off the print statemnet
    :type isPrint: bool
    :return: list of datasets with the property key
    :rtype: list
  """

  path = os.path.dirname(os.path.realpath(__file__))
  f = open(os.path.join(path, "data.json"))
  data = json.load(f)
  if isPrint:
    print("Total available datasets: " + str(len(data['datasets'])))
    print("Following are the datasets based on the search parameters: ")

  # if(key=="name" or key=="all"):
  #   lst = []
  # else:
  #   lst = {}

  # if(key=="all"):
  #   for i in data['datasets']:
  #     lst.append(i)
  #     if isPrint:
  #       print(i)
  # elif(key in data['datasets'][0].keys()):
  #   for i in data['datasets']:
  #     if(key != 'name'):
  #       lst[i['name']] = i[key]
  #     else:
  #       lst.append(i[key])
  #   if isPrint:
  #     print("Key: " + key + "\n" + str(lst))
  # else:
  #   print("The key " + key + " is not a valid key")
  #   print("Valid keys: " + str(data.keys()))

  # f.close()
  # return lst

  datasets = data['datasets']
  dataset_list = []
  if(search_key == "all"):
    for i in datasets:
      if(search_term != ""):
        values = [x.lower() for x in i.values()]
        for val in values:
          if search_term.lower() in val:
            dataset_list.append(i)
            break
      else:
        dataset_list.append(i)
  else:
    dataset_keys = list_dataset_keys()
    if search_key in dataset_keys:
      for i in datasets:
        if(search_term != ""):
          if search_term.lower() in i[search_key].lower():
            dataset_list.append(i)
        else:
          dataset_list.append(i)
    else:
      raise KeyError("Invalid key.")

  f.close()
  if isPrint:
    for i in dataset_list:
      print(json.dumps(i, sort_keys=True, indent=4))
      print()
  return dataset_list